﻿using System;
using System.Threading;

namespace Trådeksempel7_MonitorSynk
{
    // Monitor-klasse. Klassen er thread-safe
    // For at undgå problemer med at objekt låses udefra og også bruges som lock i synkroniseringen inden i metoderne
    // anbefales istedet at lave en privat lås - her kan anvendes et "dummy" objekt

    public class TælleMonitorInternLock
    {
        private int taeller;			// fælles data i monitor
        private object taellerLock = new object();      // bruges som intern lås

        // Konstruktør som nulstiller tæller
        public TælleMonitorInternLock()
        {
            taeller = 0;
        }

        // Metoden er synkroniseret
        public void Optael()
        {
            lock (taellerLock)				// tag objekt-lås (synkroniser)
            {
                taeller++;
            }

            // Alternativt:
            //Interlocked.Increment(ref taeller);
        }

        // Metoden er synkroniseret
        public void Nedtael()
        {
            lock (taellerLock)
            {
                taeller--;			// tag objekt-lås (synkroniser)
            }

            // Alternativt:
            //Interlocked.Decrement(ref taeller);
        }

        public int GetTaeller()
        {
            int tallerValue;
            lock (taellerLock)
            {
                tallerValue = taeller;			// tag objekt-lås (synkroniser)
            }
            return tallerValue;			// synkronisering er ikke nødvendig, når tæller kun læses og er simpel type, der hentes i en maskin-instruktion
        }
    }
}
